@@ -73,7 +73,7 @@ impl FdReadableSet {
     /// Add an fd to the set. The fd is ignored if negative (for convenience).
     pub fn add(&mut self, fd: RawFd) {
         if fd >= (libc::FD_SETSIZE as RawFd) {
-            //FLOGF(error, "fd %d too large for select()", fd);
+            //flogf!(error, "fd %d too large for select()", fd);
             return;
         }
         if fd >= 0 {
@@ -103,9 +103,7 @@ impl FdReadableSet {
                 &mut tvs
             }
         };
-        unsafe {
-            return libc::select(self.nfds_, &mut self.fdset_, null, null, timeout);
-        }
+        unsafe { libc::select(self.nfds_, &mut self.fdset_, null, null, timeout) }
     }
 
     /// Check if a single fd is readable, with a given timeout.
@@ -117,13 +115,13 @@ impl FdReadableSet {
         let mut s = Self::new();
         s.add(fd);
         let res = s.check_readable(timeout);
-        return res > 0 && s.test(fd);
+        res > 0 && s.test(fd)
     }
 
     /// Check if a single fd is readable, without blocking.
     /// Returns `true` if readable, `false` if not.
     pub fn poll_fd_readable(fd: RawFd) -> bool {
-        return Self::is_fd_readable(fd, Timeout::ZERO);
+        Self::is_fd_readable(fd, Timeout::ZERO)
     }
 }
 
@@ -181,19 +179,19 @@ impl FdReadableSet {
             debug_assert_eq!(pollfd.fd, fd);
             return pollfd.revents & (libc::POLLIN | libc::POLLHUP) != 0;
         }
-        return false;
+        false
     }
 
     fn do_poll(fds: &mut [libc::pollfd], timeout: Timeout) -> c_int {
         let count = fds.len();
         assert!(count <= libc::nfds_t::MAX as usize, "count too big");
-        return unsafe {
+        unsafe {
             libc::poll(
                 fds.as_mut_ptr(),
                 count as libc::nfds_t,
                 timeout.as_poll_msecs(),
             )
-        };
+        }
     }
 
     /// Call poll(). Note this destructively modifies the set. Return the result of poll().
@@ -201,7 +199,7 @@ impl FdReadableSet {
         if self.pollfds_.is_empty() {
             return 0;
         }
-        return Self::do_poll(&mut self.pollfds_, timeout);
+        Self::do_poll(&mut self.pollfds_, timeout)
     }
 
     /// Check if a single fd is readable, with a given timeout.
@@ -216,12 +214,12 @@ impl FdReadableSet {
             revents: 0,
         };
         let ret = Self::do_poll(std::slice::from_mut(&mut pfd), timeout);
-        return ret > 0 && (pfd.revents & libc::POLLIN) != 0;
+        ret > 0 && (pfd.revents & libc::POLLIN) != 0
     }
 
     /// Check if a single fd is readable, without blocking.
     /// Return true if readable, false if not.
     pub fn poll_fd_readable(fd: RawFd) -> bool {
-        return Self::is_fd_readable(fd, Timeout::ZERO);
+        Self::is_fd_readable(fd, Timeout::ZERO)
     }
 }
