@@ -1,18 +1,18 @@
 use crate::common::wcs2bytes;
-use crate::wchar::prelude::*;
+use crate::prelude::*;
 use crate::wildcard::wildcard_match;
 use crate::wutil::write_to_fd;
-use crate::{parse_util::parse_util_unescape_wildcards, wutil::wwrite_to_fd};
+use crate::{parse_util::parse_util_unescape_wildcards, wutil::unescape_bytes_and_write_to_fd};
 use libc::c_int;
 use std::sync::atomic::{AtomicI32, Ordering};
 
 #[rustfmt::skip::macros(category)]
 pub mod categories {
     use super::wstr;
-    use crate::wchar::prelude::*;
+    use crate::prelude::*;
     use std::sync::atomic::AtomicBool;
 
-    pub struct category_t {
+    pub struct Category {
         pub name: &'static wstr,
         pub description: LocalizableString,
         pub enabled: AtomicBool,
@@ -24,7 +24,8 @@ pub mod categories {
         (
             ($var:ident, $name:literal, $description:literal, $enabled:expr)
         ) => {
-            pub static $var: category_t = category_t {
+            #[allow(non_upper_case_globals)]
+            pub static $var: Category = Category {
                 name: L!($name),
                 description: localizable_string!($description),
                 enabled: AtomicBool::new($enabled),
@@ -61,7 +62,7 @@ pub mod categories {
             )*
 
             // Define a function which gives you a Vector of all categories.
-            pub fn all_categories() -> Vec<&'static category_t> {
+            pub fn all_categories() -> Vec<&'static Category> {
                 vec![
                     $(
                         & category_name!($cats),
@@ -146,7 +147,7 @@ pub mod categories {
     );
 }
 
-/// FLOG formats values. By default we would like to use Display, and fall back to Debug.
+/// flog formats values. By default we would like to use Display, and fall back to Debug.
 /// However that would require specialization. So instead we make two "separate" traits, bring them both in scope,
 /// and let Rust figure it out.
 /// Clients can opt a Debug type into Floggable by implementing FloggableDebug:
@@ -167,7 +168,7 @@ impl FloggableDisplay for WString {
 
 impl FloggableDisplay for &wstr {
     fn to_flog_str(&self) -> Vec<u8> {
-        wcs2bytes(self)
+        wcs2bytes(*self)
     }
 }
 
@@ -206,7 +207,7 @@ pub trait FloggableDebug: std::fmt::Debug {
     }
 }
 
-/// Write to our FLOG file.
+/// Write to our flog file.
 pub fn flog_impl(s: &[u8]) {
     let fd = get_flog_file_fd();
     if fd < 0 {
@@ -217,7 +218,7 @@ pub fn flog_impl(s: &[u8]) {
 
 /// The entry point for flogging.
 #[macro_export]
-macro_rules! FLOG {
+macro_rules! flog {
     ($category:ident, $($elem:expr),+ $(,)*) => {
         if $crate::flog::categories::$category.enabled.load(std::sync::atomic::Ordering::Relaxed) {
             #[allow(unused_imports)]
@@ -239,9 +240,9 @@ macro_rules! FLOG {
 }
 
 #[macro_export]
-macro_rules! FLOGF {
+macro_rules! flogf {
     ($category:ident, $fmt: expr, $($elem:expr),+ $(,)*) => {
-        $crate::flog::FLOG!($category, $crate::wutil::sprintf!($fmt, $($elem),*))
+        $crate::flog::flog!($category, $crate::wutil::sprintf!($fmt, $($elem),*))
     }
 }
 
@@ -254,7 +255,7 @@ macro_rules! should_flog {
     };
 }
 
-pub use {FLOG, FLOGF, should_flog};
+pub use {flog, flogf, should_flog};
 
 /// For each category, if its name matches the wildcard, set its enabled to the given sense.
 fn apply_one_wildcard(wc_esc: &wstr, sense: bool) {
@@ -302,5 +303,5 @@ pub fn get_flog_file_fd() -> c_int {
 }
 
 pub fn log_extra_to_flog_file(s: &wstr) {
-    wwrite_to_fd(s, get_flog_file_fd());
+    unescape_bytes_and_write_to_fd(s, get_flog_file_fd());
 }
