@@ -1,19 +1,18 @@
 use crate::common::{Named, escape, get_by_sorted_name};
 use crate::env::Environment;
-use crate::flog::FLOG;
+use crate::flog::flog;
 use crate::global_safety::RelaxedAtomicBool;
 use crate::input_common::{
     CharEvent, CharInputStyle, ImplicitEvent, InputEventQueuer, KeyMatchQuality,
     R_END_INPUT_FUNCTIONS, ReadlineCmd, match_key_event_to_key,
 };
 use crate::key::{self, Key, Modifiers, canonicalize_raw_escapes, ctrl};
+use crate::prelude::*;
 use crate::reader::{Reader, reader_reset_interrupted};
 use crate::threads::assert_is_main_thread;
-use crate::wchar::prelude::*;
-use once_cell::sync::Lazy;
 use std::mem;
 use std::sync::{
-    Mutex, MutexGuard,
+    LazyLock, Mutex, MutexGuard,
     atomic::{AtomicU32, Ordering},
 };
 
@@ -118,6 +117,7 @@ const INPUT_FUNCTION_METADATA: &[InputFunctionMetadata] = &[
     make_md(L!("backward-kill-path-component"), ReadlineCmd::BackwardKillPathComponent),
     make_md(L!("backward-kill-token"), ReadlineCmd::BackwardKillToken),
     make_md(L!("backward-kill-word"), ReadlineCmd::BackwardKillWord),
+    make_md(L!("backward-path-component"), ReadlineCmd::BackwardPathComponent),
     make_md(L!("backward-token"), ReadlineCmd::BackwardToken),
     make_md(L!("backward-word"), ReadlineCmd::BackwardWord),
     make_md(L!("begin-selection"), ReadlineCmd::BeginSelection),
@@ -151,6 +151,7 @@ const INPUT_FUNCTION_METADATA: &[InputFunctionMetadata] = &[
     make_md(L!("forward-char-passive"), ReadlineCmd::ForwardCharPassive),
     make_md(L!("forward-jump"), ReadlineCmd::ForwardJump),
     make_md(L!("forward-jump-till"), ReadlineCmd::ForwardJumpTill),
+    make_md(L!("forward-path-component"), ReadlineCmd::ForwardPathComponent),
     make_md(L!("forward-single-char"), ReadlineCmd::ForwardSingleChar),
     make_md(L!("forward-token"), ReadlineCmd::ForwardToken),
     make_md(L!("forward-word"), ReadlineCmd::ForwardWord),
@@ -173,6 +174,7 @@ const INPUT_FUNCTION_METADATA: &[InputFunctionMetadata] = &[
     make_md(L!("kill-bigword"), ReadlineCmd::KillBigword),
     make_md(L!("kill-inner-line"), ReadlineCmd::KillInnerLine),
     make_md(L!("kill-line"), ReadlineCmd::KillLine),
+    make_md(L!("kill-path-component"), ReadlineCmd::KillPathComponent),
     make_md(L!("kill-selection"), ReadlineCmd::KillSelection),
     make_md(L!("kill-token"), ReadlineCmd::KillToken),
     make_md(L!("kill-whole-line"), ReadlineCmd::KillWholeLine),
@@ -225,8 +227,8 @@ pub struct InputMappingSet {
 
 /// Access the singleton input mapping set.
 pub fn input_mappings() -> MutexGuard<'static, InputMappingSet> {
-    static INPUT_MAPPINGS: Lazy<Mutex<InputMappingSet>> =
-        Lazy::new(|| Mutex::new(InputMappingSet::default()));
+    static INPUT_MAPPINGS: LazyLock<Mutex<InputMappingSet>> =
+        LazyLock::new(|| Mutex::new(InputMappingSet::default()));
     INPUT_MAPPINGS.lock().unwrap()
 }
 
@@ -438,7 +440,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
         // Use either readch or readch_timed, per our param.
         if self.idx == self.peeked.len() {
             let newevt = if escaped {
-                FLOG!(reader, "reading timed escape");
+                flog!(reader, "reading timed escape");
                 match self.event_queue.readch_timed_esc() {
                     Some(evt) => evt,
                     None => {
@@ -447,7 +449,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                     }
                 }
             } else {
-                FLOG!(reader, "readch timed sequence key");
+                flog!(reader, "readch timed sequence key");
                 match self.event_queue.readch_timed_sequence_key() {
                     Some(evt) => evt,
                     None => {
@@ -456,7 +458,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                     }
                 }
             };
-            FLOG!(reader, format!("adding peeked {:?}", newevt));
+            flog!(reader, format!("adding peeked {:?}", newevt));
             self.peeked.push(newevt);
         }
         // Now we have peeked far enough; check the event.
@@ -466,7 +468,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
         if kevt.seq == L!("\x1b") && key.modifiers == Modifiers::ALT {
             self.idx += 1;
             self.subidx = 0;
-            FLOG!(reader, "matched delayed escape prefix in alt sequence");
+            flog!(reader, "matched delayed escape prefix in alt sequence");
             return self.next_is_char(style, Key::from_raw(key.codepoint), true);
         }
         if *style == KeyNameStyle::Plain {
@@ -474,7 +476,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
             if let Some(key_match) = &result {
                 assert!(self.subidx == 0);
                 self.idx += 1;
-                FLOG!(reader, "matched full key", key, "kind", key_match);
+                flog!(reader, "matched full key", key, "kind", key_match);
             }
             return result;
         }
@@ -487,7 +489,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                     self.idx += 1;
                     self.subidx = 0;
                 }
-                FLOG!(
+                flog!(
                     reader,
                     format!(
                         "matched char {} with offset {} within raw sequence of length {}",
@@ -502,11 +504,11 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                 if self.subidx + 1 == actual_seq.len() {
                     self.idx += 1;
                     self.subidx = 0;
-                    FLOG!(reader, "matched escape prefix in raw escape sequence");
+                    flog!(reader, "matched escape prefix in raw escape sequence");
                     return self.next_is_char(style, Key::from_raw(key.codepoint), true);
                 } else if actual_seq
                     .get(self.subidx + 1)
-                    .cloned()
+                    .copied()
                     .map(|c| Key::from_single_char(c).codepoint)
                     == Some(key.codepoint)
                 {
@@ -515,7 +517,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                         self.idx += 1;
                         self.subidx = 0;
                     }
-                    FLOG!(reader, format!("matched {key} against raw escape sequence"));
+                    flog!(reader, format!("matched {key} against raw escape sequence"));
                     return Some(KeyMatchQuality::Exact);
                 }
             }
@@ -570,7 +572,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
             prev = *key;
         }
         if self.subidx != 0 {
-            FLOG!(
+            flog!(
                 reader,
                 "legacy binding matched prefix of key encoding but did not consume all of it"
             );
@@ -619,7 +621,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
                 continue;
             }
 
-            // FLOG!(reader, "trying mapping", format!("{:?}", m));
+            // flog!(reader, "trying mapping", format!("{:?}", m));
             if self.try_peek_sequence(&m.key_name_style, &m.seq, &mut quality) {
                 // // A binding for just escape should also be deferred
                 // // so escape sequences take precedence.
@@ -647,7 +649,7 @@ impl<'q, Queuer: InputEventQueuer + ?Sized> EventQueuePeeker<'q, Queuer> {
         }
         if self.char_sequence_interrupted() {
             // We might have matched a longer sequence, but we were interrupted, e.g. by a signal.
-            FLOG!(reader, "torn sequence, rearranging events");
+            flog!(reader, "torn sequence, rearranging events");
             return None;
         }
 
@@ -725,7 +727,7 @@ impl<'a> Reader<'a> {
                     return evt;
                 }
                 CharEvent::Key(ref kevt) => {
-                    FLOG!(
+                    flog!(
                         reader,
                         "Read char",
                         kevt.key,
@@ -751,7 +753,7 @@ impl<'a> Reader<'a> {
         // Check for ordinary mappings.
         let ip = input_mappings();
         if let Some(mapping) = peeker.find_mapping(vars, &ip) {
-            FLOG!(
+            flog!(
                 reader,
                 format!("Found mapping {:?} from {:?}", &mapping, &peeker.peeked)
             );
@@ -771,7 +773,7 @@ impl<'a> Reader<'a> {
             return;
         }
 
-        FLOG!(reader, "no generic found, ignoring char...");
+        flog!(reader, "no generic found, ignoring char...");
         let _ = peeker.next();
         peeker.consume();
     }
@@ -970,7 +972,7 @@ mod tests {
     use crate::env::EnvStack;
     use crate::input_common::{CharEvent, InputData, InputEventQueuer, KeyEvent};
     use crate::key::Key;
-    use crate::wchar::prelude::*;
+    use crate::prelude::*;
 
     struct TestInputEventQueuer {
         input_data: InputData,
